// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import AppleExtensions

extension Color {
    func uiColor() -> UIColor {
        if #available(iOS 14.0, *) {
            return UIColor(self)
        }

        let scanner: Scanner = Scanner(string: description.trimmingCharacters(in: CharacterSet.alphanumerics.inverted))
        var hexNumber: UInt64 = 0
        var r: CGFloat = 0.0, g: CGFloat = 0.0, b: CGFloat = 0.0, a: CGFloat = 0.0

        let result: Bool = scanner.scanHexInt64(&hexNumber)
        if result {
            r = CGFloat((hexNumber & UInt64(0xFF000000)) >> UInt64(24)) / CGFloat(255)
            g = CGFloat((hexNumber & UInt64(0x00FF0000)) >> UInt64(16)) / CGFloat(255)
            b = CGFloat((hexNumber & UInt64(0x0000FF00)) >> UInt64(8)) / CGFloat(255)
            a = CGFloat(hexNumber & UInt64(0x000000FF)) / CGFloat(255)
        }
        return UIColor(red: r, green: g, blue: b, alpha: a)
    }

    private static var seededColors: [Color] = [
        R.color.identity.color1.color(),
        R.color.identity.color2.color(),
        R.color.identity.color3.color(),
        R.color.identity.color4.color(),
        R.color.identity.color5.color(),
        R.color.identity.color6.color()
    ]
    static func colorBySeed(_ seed: String?, colors: [Color] = seededColors) -> Color {
        let fallbackColor = colors.first ?? R.color.primary.color()
        guard let seed = seed else {
            return fallbackColor
        }
        let seedIndex: Int = abs(bkdrHash(seed)) % max(colors.count, 1)
        let color: Color
        if seed.isEmpty {
            color = fallbackColor
        } else if let seededColor = colors[safe: seedIndex] {
            color = seededColor
        } else {
            color = fallbackColor
        }
        return color
    }

    private static let seed: Double = 131
    private static let seed2: Double = 137.0
    private static let maxSafeInteger = 9007199254740991.0 / seed2

    private static func bkdrHash(_ string: String) -> Int {
        var hash: Double = 0
        for char in "\(string)x" {
            if let scl = String(char).unicodeScalars.first?.value {
                if hash > maxSafeInteger {
                    hash /= seed2
                }
                hash = hash * seed + Double(scl)
            }
        }
        return Int(round(hash))
    }
}
