// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

extension View {

    @ViewBuilder
    func alert<E : Error>(for error: Binding<E?>, okAction: (() -> Void)? = nil) -> some View {
        alertOnBackground(
            isPresented: Binding(
                get: { error.wrappedValue != nil },
                set: { isPresented in
                    if !isPresented && error.wrappedValue != nil {
                        error.wrappedValue = nil
                    }
                }),
            alert: { Alert.error(error.wrappedValue, okAction: okAction) }
        )
    }

    func notImplementedAlert(isPresented: Binding<Bool>) -> some View {
        self.alertOnBackground(isPresented: isPresented, alert: {
            Alert(
                title: Text("Ikke implementeret"),
                message: Text("Denne feature er endnu ikke implementeret."),
                dismissButton: .default(Text(Localizable.common_ok()))
            )
        })
    }

    @ViewBuilder
    func applyDefaultShadow() -> some View {
        shadow(color: R.color.shadowSolid.color().opacity(0.3), radius: 8, x: 0, y: 2)
    }

    @ViewBuilder
    func applyDefaultCardShadow(yOffset: CGFloat = 0) -> some View {
        shadow(color: R.color.cardShadow.color().opacity(0.7), radius: 5, x: 0, y: yOffset)
    }

    /// Positions this view within an invisible frame with the specified size.
    /// - Parameters:
    ///   - size: A fixed size for the resulting view.
    ///   - alignment: The alignment of this view inside the resulting view. alignment applies if this view is smaller than the size given by the resulting frame.
    /// - Returns: A view with fixed dimensions of size.
    @ViewBuilder
    func frame(size: CGFloat, alignment: Alignment = .center) -> some View {
        frame(width: size, height: size, alignment: alignment)
    }

    @ViewBuilder
    func hidden(_ hide: Bool) -> some View {
        if hide {
            hidden()
        } else {
            self
        }
    }

    /// Multiple sheets on the same view, does not work unless you put them on different views.
    /// This is a workaround to allow multiple sheets on the same view, by adding empty views as background with sheets.
    @ViewBuilder
    func sheetOnBackground<Content: View>(isPresented: Binding<Bool>, content: @escaping () -> Content) -> some View {
        background(
            EmptyView()
                .sheet(isPresented: isPresented, content: content)
        )
    }

    /// Multiple alerts on the same view, does not work unless you put them on different views.
    /// This is a workaround to allow multiple alerts on the same view, by adding empty views as background with sheets.
    @ViewBuilder
    func alertOnBackground(isPresented: Binding<Bool>, alert: @escaping () -> Alert) -> some View {
        background(
            EmptyView()
                .alert(isPresented: isPresented, content: alert)
        )
    }
}
