// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SwiftUI
import MaternityServices
import ConfigurationServices
import GMAppData
import UserSettingsServices
import ErrorData
import CoreData

/// There are a lot of different views, who needs knowledge of the maternity data.
/// Therefor this is intended to be used as an EnvironmentObject.
/// It is a singleton, such that view models can initiate update requests without having access to the instance as an EnvironmentObject.
/// As long as we are using this class and try to avoid the direct use of the services in other places, then all views will receive the data updates.
final class MaternityDataEnvironmentStoreImpl : ObservableObject, MaternityDataEnvironmentStore {
    @DI private var logger: Logger
    @DI private var localStorageService: MaternityStorageService

    @Published private (set) var selectedMaternity: SelectedMaternity?
    
    var dateOfBirthForSelectedMaternity: Date? {
        selectedMaternity?.dateOfBirth
    }
    
    var currentWeekForSelectedMaternity: Int? {
        selectedMaternity?.week
    }

    static let shared = MaternityDataEnvironmentStoreImpl()

    private init() { }

    func changeSelectedMaternity(_ maternity: SelectedMaternity) -> AnyPublisher<Void, Never> {
        if selectedMaternity == nil {
            return localStorageService.saveMaternity(maternity: maternity.toRepoModel())
                .receiveOnMain()
                .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel) in
                    let repoModel = maternity.toRepoModel()
                    self?.selectedMaternity = repoModel.toSelectedMaternity()
                    NotificationCenter.default
                        .post(notification: .TabBarController.changeOfSelectedMaternity, value: "")
                })
                .mapToVoid()
                .replaceError(with: Void())
                .eraseToAnyPublisher()
        }
        return localStorageService.updateMaternity(maternity: maternity.toRepoModel())
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self?.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .replaceError(with: Void())
            .eraseToAnyPublisher()
    }
    
    func changeDateOfBirthForSelectedMaternity(_ date: Date?) -> AnyPublisher<Void, ServiceError> {
        guard let selectedMaternity = selectedMaternity else {
            return Just(Void())
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        let maternityy = selectedMaternity.toRepoModel(dateOfBirth: date)
        return localStorageService
            .updateMaternity(maternity: maternityy)
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self?.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func requestUpdate() -> AnyPublisher<Void, ServiceError> {
        localStorageService.fetchMaternity()
            .receiveOnMain()
            .handleEvents(receiveOutput: { (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .eraseToAnyPublisher()
    }

    func clear() {
        selectedMaternity = nil
        localStorageService.deleteMaternity()
    }
}
