// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import CMSData

struct ArticleDetailView: View, ScreenTrackable {
    let article : Article
    let highlightSectionId: Int?
    @State private var didStartHighlightScroll: Bool = false
    @State private var didAppear: Bool = false

    @State private var isArticleDescriptionLoading: Bool = true
    @State private var articleDescriptionHeight: CGFloat = 50

    private let dateFormatter: DateFormatter

    init(article: Article, highlightSectionId: Int? = nil) {
        self.article = article
        self.highlightSectionId = highlightSectionId
        self.dateFormatter = DateFormatter()
        self.dateFormatter.timeStyle = .none
        self.dateFormatter.dateStyle = .medium
    }

    var body: some View {
        ScrollView {
            if #available(iOS 14, *) {
                ScrollViewReader { reader in
                    content
                        .onAppear(perform: {
                            scrollToHighlightedSectionIfNeeded(reader: reader)
                        })
                }
            } else {
                content
            }
        }
        .background(R.color.viewBackground.color())
        .navigationBarHidden(false)
        .navigationBarBackButtonHidden(false)
        .navigationBarTitle(Text(article.title), displayMode: .inline)
        .screenTracking(screenTrackingName)
        .onAppear(perform: {
            // This is needed avoid crash on scroll to highlighted on iOS 14 🤯
            self.didAppear = true
        })
    }

    private var content: some View {
        VStack(alignment: .leading, spacing: 16) {
            Group {
                Group {
                    if let content = article.content {
                        ZStack {
                            HTMLView(dynamicHeight: $articleDescriptionHeight,
                                     isLoading: $isArticleDescriptionLoading,
                                     htmlFilePath: R.file.articleContentHtml()!,
                                     html: content,
                                     fadeInOnLoad: true,
                                     scrollEnabled: false)

                            if isArticleDescriptionLoading {
                                HStack {
                                    Spacer()
                                    ActivityIndicatorView(isAnimating: isArticleDescriptionLoading,
                                                          style: .medium,
                                                          hidesWhenStopped: true)
                                    Spacer()
                                }
                            }
                        }
                        .frame(height: articleDescriptionHeight)
                        .padding(.vertical)
                    } else if let description = article.description { // Fallback to normal description, if there is not content: https://jira.nspop.dk/browse/DGAF-2209
                        Text(description)
                            .applyTextStyle(fontStyle: .body)
                            .padding(.vertical)
                    }
                }
                ForEach(article.sections) { section in
                    ArticleSectionCard(
                        articleSection: section,
                        autoExpand: highlightSectionId == section.id
                    )
                        .id(section.id)
                }

                HStack {
                    Spacer()
                    Text(Localizable.article_last_updated_on(dateFormatter.string(from: article.lastUpdated)))
                        .applyTextStyle(fontStyle: .info, textColor: R.color.greyText.color())
                    Spacer()
                }
            }
            .padding(.horizontal)
            // Using group to apply padding, instead of directly on the VStack, since iOS 15 scrolling is completely broken when setting padding on VStack... 🤯
        }
        .padding(.vertical)
    }

    @available(iOS 14.0, *)
    private func scrollToHighlightedSectionIfNeeded(reader: ScrollViewProxy) {
        guard let highlightedId = highlightSectionId, !didStartHighlightScroll else { return }

        DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(300)) {
            withAnimation {
                reader.scrollTo(highlightedId, anchor: .top)
            }
        }
        didStartHighlightScroll = true
    }
}

struct ArticleDetailView_Previews: PreviewProvider {
    static var previews: some View {
        NavigationView {
            ArticleDetailView(
                article: CMSData.Category.mockList().first!.articles.first!
            )
        }
    }
}
