// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine
import CMSData

struct ArticleCategoryCard: View {
    private let imageFrameSize: CGFloat = 50

    let title: String
    let description: String?
    let iconUrl: URL?
    let infoText: String?

    init(title: String, description: String?, iconUrl: URL?, infoText: String?) {
        self.title = title
        self.description = description
        self.iconUrl = iconUrl
        self.infoText = infoText
    }

    init(category: CMSData.Category) {
        self.init(
            title: category.name,
            description: category.description,
            iconUrl: category.iconUrl,
            infoText: category.numberOfArticlesText
        )
    }

    var body: some View {
        BasicCardView {
            HStack(alignment: .top) {
                CircleCardView {
                    URLImage(
                        url: iconUrl ?? URL(fileURLWithPath: ""),
                        imageView: { uiImage in
                            Image(uiImage: uiImage)
                                .renderingMode(.original)
                                .resizable()
                                .aspectRatio(contentMode: .fit)
                                .frame(size: imageFrameSize - 20)
                                .clipped()
                        },
                        placeholderView: {
                            ActivityIndicatorView(
                                isAnimating: true,
                                style: .medium,
                                hidesWhenStopped: true
                            )
                        },
                        errorView: { _ in
                            Circle().fill(Color(UIColor.lightGray))
                        }
                    )
                }
                .frame(size: imageFrameSize)

                NavigationTitleVStack(
                    title: title,
                    text: description,
                    infoText: infoText
                )
                .padding(.leading, 10)
            }
            .padding()
        }
    }
}

struct ArticleCategoryCard_Previews: PreviewProvider {

    static var previews: some View {
        ScrollView {
            VStack {
                ForEach(CMSData.Category.mockList()) { category in
                    ArticleCategoryCard(category: category)
                }
            }
            .padding()
        }
        .background(R.color.viewBackground.color())
    }
}
