// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI

struct KnowledgeViewChecklistSection: View {
    @EnvironmentObject var environmentStore: KnowledgeEnvironmentStoreImpl
    @EnvironmentObject var maternityStore: MaternityDataEnvironmentStoreImpl
    @DI private var userContext: UserContextProtocol

    private var hasValidMaternity: Bool {
        maternityStore.selectedMaternity != nil
    }

    var body: some View {
        KnowledgeViewListSection(title: Localizable.checklists_title()) {
            Group {
                    if let error = environmentStore.error {
                        ErrorStateView(
                            text: error.localizedDescription,
                            retryAction: environmentStore.fetchChecklists
                        )
                    } else {
                        ScrollView(.horizontal, showsIndicators: false) {
                            HStack(spacing: 8) {
                                ForEach(environmentStore.checklists) { checklist in
                                    NavigationLink(
                                        destination: ChecklistDetailView(checklist: checklist),
                                        label: {
                                            HorizontalChecklistView(checklist: checklist)
                                        }
                                    )
                                    .tag(checklist.id)
                                }
                            }
                            .padding()
                            .frame(minHeight: 100)
                        }
                        .opacity(environmentStore.isLoading ? 0.0 : 1.0)
                        .overlay(
                            ActivityIndicatorView(
                                isAnimating: environmentStore.isLoading,
                                color: R.color.primary()!,
                                style: .medium,
                                hidesWhenStopped: true
                            )
                        )
                        .transition(.opacity)
                        .animation(.easeOut(duration: 0.2))
                    }
            }
        }
        .protectedAppearanceEvents(onAppear: { environmentStore.fetchChecklists() }
        )
    }
}

#if DEBUG
struct KnowledgeViewChecklistSection_Previews: PreviewProvider {
    static var previews: some View {
        KnowledgeViewChecklistSection()
            .environmentObject(KnowledgeEnvironmentStoreImpl.preview)
    }
}
#endif
