// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI
import CMSServices

extension SettingsView {
    final class ViewModel: ObservableObject {
        @DI var localAuthentication: LocalAuthenticationContext
        @DI var userContext: UserContextProtocol
        @DI var pagesServices: PagesService
        @DI var maternityEnvironmentStore: MaternityDataEnvironmentStore

        @Published var shouldInitiateBiometricEnabling: Bool = false
        @Published var showAlertToInitiateBiometricEnabling: Bool = false

        private var cancelBag = CancelBag()

        /// This is not a `Published` variable to make sure that we as the developers have control over this.
        /// The user sometimes toggles this, but it is also controlled by the current state of the TIM library,
        /// which makes this a little complicated. This value is used as value inside a custom `Binding` instance.
        var isBiometricToggleSwitchOn: Bool = false

        var isBiometricEnabled: Bool { false }

        var isBiometricAvailable: Bool {
            localAuthentication.isBiometricAvailable
        }

        var biometricName: String {
            localAuthentication.biometricName
        }

        var appVersion: String {
            Bundle.main.version ?? Localizable.common_missing_data()
        }

        var appBuildNumber: String {
            Bundle.main.buildNumber ?? Localizable.common_missing_data()
        }

        let title: String = Localizable.more_settings_title()

        init() {
            updateValues()
            observeChanges()
        }

        func updateValues() {
            isBiometricToggleSwitchOn = isBiometricEnabled
            objectWillChange.send() /// Since the `isBiometricToggleSwitchOn` isn't `Published`, we need to publish this change by hand.
        }

        private func observeChanges() {
            $shouldInitiateBiometricEnabling
                .removeDuplicates()
                .sink { [weak self] isPresenting in
                    if !isPresenting {
                        self?.updateValues()
                    }
                }
                .store(in: &cancelBag)
        }
    }
}
