// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct OverviewView: View, ScreenTrackable {
    @ObservedObject var viewModel = ViewModel()
    @EnvironmentObject var maternityEnvironmentStore: MaternityDataEnvironmentStoreImpl
    @State private var didAppear: Bool = false
    @State private var scrollOffset: CGPoint = .zero
    @DI private var logger: Logger

    var navigationBarHidden: Bool {
        if #available(iOS 14, *) {
            return true
        } else {
            return didAppear // This weird stuff is needed to make iOS 13 behave correctly, but it makes iOS 14 behave incorrectly... 🤦🏻‍♂️
        }
    }

    var body: some View {
        NavigationView {
            RefreshableScrollView {
                viewModel.fetchData(force: true, onRefreshComplete: $0)
            } content: {
                VStack(spacing: 20) {
                    welcomeHeaderView
                        .padding()
                    headerView
                    if viewModel.showArticlesSection {
                        DataLoadingView(
                            state: viewModel.state,
                            retryAction: { viewModel.fetchData() },
                            content: {
                                VStack(spacing: 20) {
                                    if viewModel.showCurrentWeekArticles {
                                        OverviewKnowledgeSectionView(
                                            title: viewModel.currentRelevantArticleTitle,
                                            actionTitle: Localizable.overview_find_more_knowledge_button(),
                                            action: {
                                                viewModel.pushKnowledgeSection = true
                                            },
                                            categories: viewModel.currentRelevantArticleResults
                                        )
                                    }
                                    if viewModel.showNextWeekArticles {
                                        OverviewKnowledgeSectionView(
                                            title: Localizable.overview_relevant_next_week_title(),
                                            actionTitle: nil,
                                            action: nil,
                                            categories: viewModel.nextRelevantArticleResults
                                        )
                                    }
                                }
                            }
                        )
                            .padding(.horizontal)
                    }
                }
                .background(
                    VStack(spacing: 0) {
                        // Using screen size instead of `GeometryReader` because it ruins the publishing of viewModel changes on iOS 13...
                        BubblesBackgroundView(size: UIScreen.main.bounds.size)
                        Spacer()
                    }
                )
                NavigationLink(
                    destination: KnowledgeContentView(preferredNavigationBarDisplayMode: .inline)
                        .navigationBarHidden(false)
                        .navigationBarBackButtonHidden(false),
                    isActive: $viewModel.pushKnowledgeSection,
                    label: {
                        EmptyView()
                    }
                )
            }
            .overlay(
                GeometryReader { geo in
                    VStack(spacing: 0) {
                        VisualBlurEffectView(style: .systemChromeMaterial)
                            .frame(height: geo.safeAreaInsets.top)
                        DefaultDivider()
                        Spacer(minLength: 0)
                    }
                    .opacity(Double(max(min(-scrollOffset.y / geo.safeAreaInsets.top, 1.0), 0.0)))
                    .edgesIgnoringSafeArea(.top)
                }
            )
            .navigationBarTitle("", displayMode: .inline)
            .navigationBarHidden(navigationBarHidden)
            .navigationBarBackButtonHidden(navigationBarHidden)
            .screenTracking(screenTrackingName)
            .protectedAppearanceEvents(onAppear: { viewModel.fetchData() })
            .protectedAppearanceEvents(
                onAppear: { didAppear = true },
                onDisappear: { didAppear = false }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentDidGiveBirthModal,
                content: {
                    OverviewDateOfBirthView(viewModel: viewModel.dateOfBirthViewModel)
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentDateOfDeliveryPicker,
                content: {
                    OverviewDateOfDeliveryView(viewModel: viewModel.dateOfDeliveryViewModel,
                                               action: { viewModel.presentCalculateDateOfDeliveryPicker = true })
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentCalculateDateOfDeliveryPicker,
                content: {
                    OverviewCalculateDateOfDeliveryView(viewModel: viewModel.calculateDateOfDeliveryViewModel)
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentStartNewPregnancyModal,
                content: {
                    OverviewStartNewPregnancyView(viewModel: viewModel.startNewPregnancyViewModel)
                }
            )
        }
        .navigationViewStyle(StackNavigationViewStyle())
    }
    
    @ViewBuilder
    private var welcomeHeaderView: some View {
        OverviewWelcomeHeaderView(
            title: viewModel.navigationBarTitle,
            subtitle: viewModel.navigationBarSubtitle,
            buttonAction: viewModel.headerViewButtonAction
        )
    }
    
    @ViewBuilder
    private var headerView: some View {
        switch viewModel.pregnancyState {
        case .none:
            OverviewNoCourseHeaderView(
                numberOfFetuses: 1,
                presentTermModal: $viewModel.presentDateOfDeliveryPicker
            )
                .padding(.horizontal)
        case .activeAndWaitingForBirth:
            OverviewWaitingHeaderView(
                monthsIntoPregnancy: maternityEnvironmentStore.selectedMaternity?.monthsIntoPregnancy,
                dueDate: maternityEnvironmentStore.selectedMaternity?.dateOfDelivery,
                month: Int(maternityEnvironmentStore.selectedMaternity?.monthsIntoPregnancy ?? 0) + 1, // plus one since the value is starting at 0
                week: maternityEnvironmentStore.selectedMaternity?.week ?? 0,
                gestationAge: maternityEnvironmentStore.selectedMaternity?.gestationAge ?? Localizable.common_missing_data(),
                trimester: maternityEnvironmentStore.selectedMaternity?.trimester ?? 0,
                numberOfFetuses: 1,
                didGiveBirthButtonPressed: $viewModel.presentDidGiveBirthModal,
                editPregnancyButtonPressed: $viewModel.presentDateOfDeliveryPicker
            )
                .padding(.horizontal)
        case .activeAndGaveBirth:
            if let date = maternityEnvironmentStore.dateOfBirthForSelectedMaternity {
                OverviewGaveBirthStillActiveHeaderView(
                    dateOfBirth: date,
                    numberOfFetuses: 1,
                    presentDateOfBirthModal: $viewModel.presentDidGiveBirthModal
                )
                    .padding(.horizontal)
            } else {
                EmptyView()
                    .onAppear(perform: {
                        logger.error("Overview is requesting header view for `activeAndGaveBirth`, but does not have a birth date. This should never happen!")
                    })
            }
        }
    }
}

struct OverviewView_Previews: PreviewProvider {
    static var previews: some View {
        OverviewView(viewModel: OverviewView.PreviewViewModel())
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
    }
}
