// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import GMAppData
import CMSServices
import CMSData
import ErrorData
import Combine

extension OverviewView {
    class ViewModel: ObservableObject {
        private var cancelBag = CancelBag()
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI private var articlesService: ArticlesService

        @Published var state: DataLoadingState = .initial
        @Published var presentDidGiveBirthModal: Bool = false
        @Published var presentStartNewPregnancyModal: Bool = false
        @Published var presentDateOfDeliveryPicker: Bool = false
        @Published var presentCalculateDateOfDeliveryPicker: Bool = false
        @Published var pushKnowledgeSection: Bool = false
        @Published var shouldReloadData: Bool = false
        
        @Published var currentRelevantArticleResults: [CMSData.Category] = []
        @Published var nextRelevantArticleResults: [CMSData.Category] = []

        var navigationBarTitle: String {
            Localizable.overview_welcome_message_unauthenticated()
        }
        
        var navigationBarSubtitle: String {
            Localizable.bottom_navigation_title_overview()
        }
        
        var headerViewButtonAction: (() -> Void)? {
            hasDateOfBirth ? { self.presentStartNewPregnancyModal = true } : nil
        }

        private var currentWeekForArticles: Int {
            guard let week = maternityEnvironmentStore.selectedMaternity?.week else {
                return 1
            }
            return week
        }

        //Has to be kept here, to avoid recreation of view model on environment changes, while sheet is present.
        let dateOfBirthViewModel = OverviewDateOfBirthView.ViewModel()
        let dateOfDeliveryViewModel = OverviewDateOfDeliveryView.ViewModel()
        let startNewPregnancyViewModel = OverviewStartNewPregnancyView.ViewModel()
        let calculateDateOfDeliveryViewModel = OverviewCalculateDateOfDeliveryView.ViewModel()

        var pregnancyState: OverviewPregnancyState {
            if maternityEnvironmentStore.selectedMaternity == nil {
                return .none
            } else if hasDateOfBirth {
                return .activeAndGaveBirth
            } else {
                return .activeAndWaitingForBirth
            }
        }

        var hasDateOfBirth: Bool {
            maternityEnvironmentStore.dateOfBirthForSelectedMaternity != nil
        }

        var showArticlesSection: Bool {
            showCurrentWeekArticles || showNextWeekArticles
        }

        var showCurrentWeekArticles: Bool {
            currentRelevantArticleResults.hasSearchResults
        }

        var showNextWeekArticles: Bool {
            nextRelevantArticleResults.hasSearchResults && pregnancyState != .none
        }
        
        var currentRelevantArticleTitle: String {
            pregnancyState == .none ? Localizable.overview_selected_topics_title() : Localizable.overview_relevant_right_now_title()
        }

        init() {
            listenForReloadEvents()
        }

        private func listenForReloadEvents() {
            dateOfBirthViewModel.$didSave
                .sink { [weak self] reload in
                    if reload {
                        self?.fetchData(force: true)
                    }
                }
                .store(in: &cancelBag)
            
            dateOfDeliveryViewModel.$didSave
                .sink { [weak self] reload in
                    if reload {
                        self?.fetchData(force: true)
                    }
                }
                .store(in: &cancelBag)
            
            startNewPregnancyViewModel.$didSave
                .sink { [weak self] reload in
                    if reload {
                        self?.fetchData(force: true)
                    }
                }
                .store(in: &cancelBag)
        }

        func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
            // We don't want to fetch articles on empty state.
            // The article-week does not change while logged in and if it's empty it will stay empty until next week (which would cause a new fetch upon new login).
            guard (state.allowsDataFetch && !state.isEmpty) || force else {
                return
            }
            
            state = force && state.allowRefresh ? .refreshing : .loading
            
            let categoriesRequest: AnyPublisher<CMSData.RecommendedCategories, ServiceError>
            
            switch pregnancyState {
            case .none:
                let weekNumber = Int.random(in: 1...10)
                categoriesRequest = articlesService.fetchArticleCategories(for: weekNumber)
            case .activeAndGaveBirth:
                let dateOfBirthWeekOffset = maternityEnvironmentStore.dateOfBirthForSelectedMaternity?.weekNumberAfterBirthDate() ?? 1
                categoriesRequest = articlesService.fetchAfterBirthArticleCategories(for: dateOfBirthWeekOffset)
            case .activeAndWaitingForBirth:
                categoriesRequest = articlesService.fetchArticleCategories(for: currentWeekForArticles)
            }
            
            categoriesRequest
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        case .finished:
                            // Always handle state as data, since knowledge content will handle empty state.
                            self?.state = .data
                        }
                        onRefreshComplete()
                    },
                    receiveValue: { [weak self] (categories) in
                        self?.currentRelevantArticleResults = categories.currentWeek
                        self?.nextRelevantArticleResults = categories.nextWeek
                    }
                )
                .store(in: &cancelBag)
        }
    }

    final class PreviewViewModel: ViewModel {
        override func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
            currentRelevantArticleResults = Array(CMSData.Category.mockList().prefix(2))
            nextRelevantArticleResults = [ CMSData.Category.mockList().last! ]
            state = .data
        }
    }
}
