// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct OverviewDateOfDeliveryView: View, ScreenTrackable {
    @ObservedObject var viewModel: ViewModel
    @Environment(\.presentationMode) var presentationMode
    
    let action: (() -> Void)

    var body: some View {
        NavigationView {
            VStack(alignment: .leading) {
                Text(viewModel.descriptionText)
                    .applyTextStyle(fontStyle: .body)
                DatePicker(
                    "",
                    selection: $viewModel.date,
                    in: viewModel.dateOfDeliveryClosedRange,
                    displayedComponents: .date
                )
                .compatibleDatePickerStyle()
                Spacer()
                
                if viewModel.hasSelectedMaterinity {
                    LoadingButton(
                        title: viewModel.primaryButtonText,
                        isLoading: viewModel.isLoadingSave,
                        action: viewModel.saveSelectedDate
                    )
                    .padding(.bottom)
                    HStack {
                        Spacer()
                        Button(
                            action: {
                                viewModel.showResetAlert = true },
                            label: { Text(Localizable.overview_end_pregnancy_button_title())
                                    .applyTextStyle(fontStyle: .h5, textColor: R.color.primary.color())
                            }
                        )
                    }
                } else {
                    LoadingButton(
                        title: viewModel.primaryButtonText,
                        isLoading: viewModel.isLoadingSave,
                        action: viewModel.saveSelectedDate
                    )
                    CustomButton(
                        title: Localizable.overview_pregnancy_calculate_expected_delivery_date_button_title(),
                        foregroundColor: .black,
                        backgroundColor: R.color.controlDeselected.color(),
                        action: {
                            presentationMode.wrappedValue.dismiss()
                            action()}
                    )
                    .padding(.bottom)
                }
            }
            .padding()
            .alert(for: $viewModel.error)
            .alertOnBackground(isPresented: $viewModel.showResetAlert, alert: {
                Alert(
                    title: Text(Localizable.overview_end_pregnancy_alert_title()),
                    message: Text(Localizable.overview_end_pregnancy_alert_description()),
                    primaryButton: .default(Text(Localizable.common_delete()), action: viewModel.resetMaternity),
                    secondaryButton: .cancel(Text(Localizable.common_regret()))
                )
            })
            .navigationBarTitle(Text(viewModel.navigationBarTitleText), displayMode: .inline)
            .navigationBarItems(trailing: CloseModalButton(action: {
                presentationMode.wrappedValue.dismiss()
            }))
            .protectedAppearanceEvents(
                onAppear: viewModel.reset,
                onDisappear: viewModel.reset
            )
            .screenTracking(screenTrackingName)
            .onReceive(viewModel.$didSave, perform: { didSave in
                if didSave {
                    presentationMode.wrappedValue.dismiss()
                }
            })
        }
    }
}

struct OverviewDateOfDeliveryView_Previews: PreviewProvider {
    static var previews: some View {
        OverviewDateOfDeliveryView(
            viewModel: OverviewDateOfDeliveryView.ViewModel(),
            action: { })
    }
}
