// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Combine
import ConfigurationServices
import ErrorData
import GMAppData
import SwiftUI

extension OverviewDateOfDeliveryView {
    final class ViewModel: ObservableObject {
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI private var userContext: UserContextProtocol
        @DI private var logger: Logger

        private var cancelBag = CancelBag()
        
        @Published var date: Date!
        @Published var isLoadingReset: Bool = false
        @Published var isLoadingSave: Bool = false
        @Published var error: Error?
        @Published var showResetAlert: Bool = false
        @Published var didSave: Bool = false
        
        var hasSelectedMaterinity: Bool {
            maternityEnvironmentStore.selectedMaternity != nil
        }
        
        var showResetButton: Bool {
            hasSelectedMaterinity
        }
        
        var primaryButtonText: String {
            hasSelectedMaterinity
            ? Localizable.overview_pregnancy_select_new_expected_delivery_date_button_title()
            : Localizable.overview_pregnancy_select_expected_delivery_date_button_title()
        }
        
        var navigationBarTitleText: String {
            hasSelectedMaterinity
            ? Localizable.overview_pregnancy_select_new_expected_delivery_date_title()
            : Localizable.overview_pregnancy_select_expected_delivery_date_title()
        }
        
        var descriptionText: String {
            hasSelectedMaterinity
            ? Localizable.overview_select_new_expected_delivery_date_description()
            : Localizable.overview_select_expected_delivery_date_description()
        }
        
        var lowerBound: Date { Date() }
        
        var upperBound: Date { Date().date(byAdding: .day, value: 280) ?? Date() }
        
        var dateOfDeliveryClosedRange: ClosedRange<Date> { lowerBound...upperBound }
        
        init() {
            self.reset()
        }

        func reset() {
            self.date = maternityEnvironmentStore.selectedMaternity?.dateOfDelivery ?? Date()
            self.didSave = false
            self.isLoadingSave = false
            self.error = nil
        }
        
        func resetMaternity() {
            userContext.clearEnvironmentStoreStates()
            didSave = true
        }

        func saveSelectedDate() {
            isLoadingSave = true
            saveDate(date: date)
        }

        private func saveDate(date: Date?) {
            guard let date = date else { return }
            let maternity = SelectedMaternity(dateOfDelivery: date)
            
            maternityEnvironmentStore
                .changeSelectedMaternity(maternity)
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        switch completion {
                        case .failure(let error):
                            self?.error = error
                            self?.didSave = false
                        case .finished:
                            self?.didSave = true
                        }
                        self?.isLoadingSave = false
                    },
                    receiveValue: { _ in }
                )
                .store(in: &cancelBag)            
        }
    }
}
