// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import UIKit

struct SUITextView: UIViewRepresentable {

    @Binding var text: String
    @Binding var hasFocus: Bool
    let placeholderText: String?

    func makeUIView(context: Context) -> UITextView {
        let textView = UITextView()
        if let placeholder = placeholderText, text.isEmpty {
            textView.textColor = R.color.secondaryValue()
            textView.text = placeholder
        } else {
            textView.text = text
        }
        let margin: CGFloat = 8
        textView.textContainerInset = UIEdgeInsets(
            top: margin,
            left: margin,
            bottom: margin,
            right: margin
        )
        textView.backgroundColor = .clear
        textView.delegate = context.coordinator
        textView.font = FontStyle.body.scaledUIFont()
        textView.keyboardType = .default
        textView.returnKeyType = .default
        textView.resignFirstResponder()
        textView.inputAccessoryView = constructToolbar(textView)

        return textView
    }

    func updateUIView(_ uiView: UITextView, context: Context) {
        uiView.tintColor = R.color.primary()
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(text: _text, hasFocus: _hasFocus, placeholderText: placeholderText)
    }

    private func constructToolbar(_ textView: UITextView) -> UIToolbar {
        let toolbarRect = CGRect(
            x: 0,
            y: 0,
            width: UIScreen.main.bounds.width,
            height: 50
        )

        let button = CloseKeyboardButton()
        button.addTarget(textView, action: #selector(UITextView.resignFirstResponder), for: .touchUpInside)

        let toolbar = UIToolbar(frame: toolbarRect)
        toolbar.items = [
            UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil),
            UIBarButtonItem(customView: button)
        ]
        toolbar.tintColor = R.color.primary()
        toolbar.barTintColor = R.color.tabbarBackground()
        toolbar.sizeToFit()
        return toolbar
    }

    final class Coordinator : NSObject, UITextViewDelegate {

        @DI var logger: Logger
        @Binding var text: String
        @Binding var hasFocus: Bool
        let placeholderText: String?

        init(text: Binding<String>, hasFocus: Binding<Bool>, placeholderText: String?) {
            _text = text
            _hasFocus = hasFocus
            self.placeholderText = placeholderText
            super.init()
        }

        // MARK: - UITextViewDelegate

        func textViewDidBeginEditing(_ textView: UITextView) {
            if textView.text == placeholderText {
                textView.text = ""
                textView.textColor = R.color.text()
            }
            hasFocus = true
        }

        func textViewDidEndEditing(_ textView: UITextView) {
            if textView.text.isEmpty {
                textView.text = placeholderText
                textView.textColor = R.color.secondaryValue()
            }
            hasFocus = false
        }

        func textViewDidChange(_ textView: UITextView) {
            text = textView.text
        }
    }
}

struct SUITextView_Previews: PreviewProvider {
    static var previews: some View {
        SUITextView(
            text: .constant("test"),
            hasFocus: .constant(false),
            placeholderText: "My placeholder"
        )
    }
}

private final class CloseKeyboardButton: UIButton {

    convenience init() {
        self.init(type: .system)
        setup()
    }

    override init(frame: CGRect) {
        super.init(frame: frame)
        setup()
    }

    required init?(coder: NSCoder) {
        super.init(coder: coder)
        setup()
    }

    private func setup() {
        let chevronImage = R.image.icons.chevron.large.down()
        let offset: CGFloat = 40
        imageEdgeInsets = UIEdgeInsets(top: 0, left: offset, bottom: 0, right: 0)
        titleEdgeInsets = UIEdgeInsets(top: 0, left: -offset, bottom: 0, right: 0)
        setImage(chevronImage, for: .normal)
        setTitle(Localizable.common_cta_close(), for: .normal)
        titleLabel?.font = FontStyle.h4.scaledUIFont()
        sizeToFit()
        frame.size.width += offset / 2.0
    }
}
