// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import DependencyInjection
import CMSServices
import ConfigurationServices
import Logger
import MaternityServices
import UserSettingsServices
import SharedServices
@testable import MinGraviditet

class KnowledgeEnvironmentStoreImplTests: XCTestCase {

    private var envStore = KnowledgeEnvironmentStoreImpl.shared

    override func setUp() {
        super.setUp()
        var cancelBag = CancelBag()

        envStore.clear()

        MockFlags.failRisk = .disabled

        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(ChecklistContentService.self, ChecklistContentServiceMock())
        Resolver.shared.registerAsSingleton(ConfigurationService.self, ConfigurationServiceMock())
        Resolver.shared.registerAsSingleton(DataCardService.self, DataCardServiceMock())
        Resolver.shared.registerAsSingleton(ChecklistService.self, ChecklistServiceImpl())
        Resolver.shared.registerAsSingleton(UserSettingsService.self, UserSettingsServiceMock())
        Resolver.shared.registerAsSingleton(MaternityDataEnvironmentStore.self, MaternityDataEnvironmentStoreImpl.shared)
        Resolver.shared.registerAsSingleton(Logger.self, OSLogger())

        // Make sure there is a selected maternity ...
        if MaternityDataEnvironmentStoreImpl.shared.selectedMaternity == nil {
            let expect = XCTestExpectation()
            MaternityDataEnvironmentStoreImpl.shared.requestUpdate()
                .sink(receiveCompletion: { _ in expect.fulfill() }, receiveValue: { _ in })
                .store(in: &cancelBag)
            wait(for: [expect], timeout: 2.0)
            XCTAssertNotNil(MaternityDataEnvironmentStoreImpl.shared.selectedMaternity)
        }
    }

    func testFetchChecklistsAndIsLoading() {
        var cancelBag = CancelBag()
        var didGetLoadingTrueValue: Bool = false
        XCTAssertEqual(0, envStore.checklists.count)

        let expectChecklists = XCTestExpectation(description: "checklists")
        envStore.$checklists
            .sink { (checklists: [Checklist]) in
                // Will be invoked with the initial value of zero at first...
                if checklists.count > 0 {
                    expectChecklists.fulfill()
                }
            }
            .store(in: &cancelBag)

        let expectLoading = XCTestExpectation(description: "isLoading")
        expectLoading.assertForOverFulfill = true
        envStore.$isLoading
            .sink { (isLoading) in
                // Expect a false loading and
                if !didGetLoadingTrueValue && isLoading {
                    didGetLoadingTrueValue = true //2. isLoading true
                } else if didGetLoadingTrueValue && !isLoading {
                    expectLoading.fulfill() // 3. isLoading false again
                } else {
                    XCTAssertFalse(isLoading) //1. isLoading false
                }
            }
            .store(in: &cancelBag)

        envStore.fetchChecklists()
        wait(for: [expectChecklists, expectLoading], timeout: 2.0)

        XCTAssertNil(envStore.error)
        XCTAssertGreaterThan(envStore.checklists.count, 0)
    }

    func testUpdateChecklist() {
        var cancelBag = CancelBag()
        let expectChecklists = XCTestExpectation(description: "checklists")
        expectChecklists.assertForOverFulfill = true

        let expectChecklistUpdatesForUpdate = XCTestExpectation(description: "updateChecklists")
        expectChecklistUpdatesForUpdate.assertForOverFulfill = true
        envStore.$checklists
            .sink { (checklists: [Checklist]) in
                if checklists.first?.items.first(where: { $0.isChecked }) != nil {
                    // Received update on isChecked change!
                    expectChecklistUpdatesForUpdate.fulfill()
                } else if checklists.count > 0 { // Will be invoked with the initial value of zero at first, and we don't care about that!
                    expectChecklists.fulfill()
                }
            }
            .store(in: &cancelBag)
        envStore.fetchChecklists()
        wait(for: [expectChecklists], timeout: 2.0)

        let checklist = envStore.checklists.first!
        var checklistItem = checklist.items.first!
        XCTAssertFalse(checklistItem.isChecked)
        checklistItem.isChecked = true
        XCTAssertTrue(checklistItem.isChecked)
        envStore.updateChecklist(checklist, checklistItem: checklistItem)
        let updatedChecklistItem = envStore.checklists.first(where: { $0.id == checklist.id })!.items.first(where: { $0.id == checklistItem.id })!
        XCTAssertTrue(updatedChecklistItem.isChecked)

        wait(for: [expectChecklistUpdatesForUpdate], timeout: 2.0) // Make sure that the checklist publisher is notified on the update call.
    }

    func testClear() {
        var cancelBag = CancelBag()
        XCTAssertEqual(0, envStore.checklists.count)

        let expectChecklists = XCTestExpectation(description: "checklists")
        envStore.$checklists
            .sink { (checklists: [Checklist]) in
                // Will be invoked with the initial value of zero at first...
                if checklists.count > 0 {
                    expectChecklists.fulfill()
                }
            }
            .store(in: &cancelBag)

        envStore.fetchChecklists()
        wait(for: [expectChecklists], timeout: 2.0)
        XCTAssertGreaterThan(envStore.checklists.count, 0)
        envStore.clear()
        XCTAssertEqual(0, envStore.checklists.count)
        XCTAssertFalse(envStore.isLoading)
        XCTAssertNil(envStore.error)
    }
}
