// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import DependencyInjection
import XCTest
import Logger
import UserSettingsServices
import ConfigurationServices
import TIM
@testable import MaternityServices
@testable import MinGraviditet

final class TabBarControllerTests: XCTestCase {
    private static let userSettingsMock = UserSettingsServiceMock(isPersistant: false)
    private static let dataCardServiceMock = DataCardServiceMock()
    private static let maternityEnvironmentStore = MaternityDataEnvironmentStoreImpl.shared
    private static let timAuthMock = TestMockTIMAuth()

    override func setUp() {
        super.setUp()
        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(TIMAuth.self, Self.timAuthMock)
        Resolver.shared.registerAsSingleton(TIMDataStorage.self, TestMockTIMDataStorage(hasBioAccess: false))
        Resolver.shared.registerAsSingleton(UserContextProtocol.self, UserContext())
        Resolver.shared.registerAsSingleton(Logger.self, OSLogger())
        Resolver.shared.registerAsSingleton(MaternityDataEnvironmentStore.self, Self.maternityEnvironmentStore)
        Resolver.shared.registerAsSingleton(DataCardService.self, Self.dataCardServiceMock)
        Resolver.shared.registerAsSingleton(ConfigurationService.self, ConfigurationServiceMock())
        Resolver.shared.registerAsSingleton(UserSettingsService.self, Self.userSettingsMock)

        // Reset
        Self.userSettingsMock.remove(key: .hasSeenNoActiveMaternityCoursesAlert)
        Self.userSettingsMock.remove(key: .hasSeenNoMaternityCoursesAlert)

        Self.timAuthMock.performOpenIDConnectLogin(presentingViewController: UIViewController(), completion: { _ in }) // Login to get authenticated
    }

    func testShowLoginAlertsForNoMaternityCourses() {
        var cancelBag = CancelBag()
        Self.dataCardServiceMock.dataCards = []

        let expect = XCTestExpectation()
        Self.maternityEnvironmentStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 1.0)

        let tabBarController = TabBarController()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)
        tabBarController.showLoginAlertsIfNeeded()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertTrue(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)
    }

    func testShowLoginAlertsForNoActiveMaternityCourses() {
        var cancelBag = CancelBag()
        Self.dataCardServiceMock.dataCards = [
            DataCardServiceMock.dataCard(status: .closed, date: Date(), episodeOfCareId: "1")
        ]
        let expect = XCTestExpectation()
        Self.maternityEnvironmentStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 20.0)

        let tabBarController = TabBarController()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)
        tabBarController.showLoginAlertsIfNeeded()
        XCTAssertTrue(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)

        Self.dataCardServiceMock.dataCards = [
            DataCardServiceMock.dataCard(status: .closed, date: Date(), episodeOfCareId: "1"),
            DataCardServiceMock.dataCard(status: .active, date: Date(), episodeOfCareId: "2")
        ]

        let expect2 = XCTestExpectation()
        Self.maternityEnvironmentStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect2.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect2], timeout: 20.0)

        tabBarController.showLoginAlertsIfNeeded()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)


        Self.dataCardServiceMock.dataCards = [
            DataCardServiceMock.dataCard(status: .new, date: Date(), episodeOfCareId: "-"),
        ]
        let expect3 = XCTestExpectation()
        Self.maternityEnvironmentStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect3.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect3], timeout: 20.0)

        tabBarController.showLoginAlertsIfNeeded()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertTrue(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)

        // Call it again to verify that result doesn't change when called again...
        tabBarController.showLoginAlertsIfNeeded()
        XCTAssertFalse(Self.userSettingsMock.get(key: .hasSeenNoActiveMaternityCoursesAlert) == true)
        XCTAssertTrue(Self.userSettingsMock.get(key: .hasSeenNoMaternityCoursesAlert) == true)
    }
}
