// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import DependencyInjection
import TIM
import Combine
import ConfigurationServices
import UserSettingsServices
import HTTPNetworking
import ErrorData
import CMSServices
import Logger
import MaternityServices

@testable import MinGraviditet

class UserContextTests: XCTestCase {

    private var userContext = UserContext()
    @Inject private var timAuth: TIMAuth
    @Inject private var timDataStorage: TIMDataStorage
    @Inject private var configurationService: ConfigurationService

    private var store: Set<AnyCancellable> = []

    override func setUpWithError() throws {
        continueAfterFailure = false

        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(Logger.self, OSLogger())
        Resolver.shared.registerAsSingleton(TIMAuth.self, TestMockTIMAuth())
        Resolver.shared.registerAsSingleton(TIMDataStorage.self, TestMockTIMDataStorage(hasBioAccess: false))
        Resolver.shared.registerAsSingleton(ChecklistContentService.self, ChecklistContentServiceMock())
        Resolver.shared.registerAsSingleton(ChecklistService.self, ChecklistServiceImpl())
        Resolver.shared.registerAsSingleton(ConfigurationService.self, ConfigurationServiceMock())
        Resolver.shared.registerAsSingleton(UserContextProtocol.self, userContext)
        Resolver.shared.registerAsSingleton(UserSettingsService.self, UserSettingsServiceMock())
        Resolver.shared.registerAsSingleton(DataCardService.self, DataCardServiceMock())
        Resolver.shared.registerAsSingleton(NetworkSession.self, URLSession.mockSession)

        Resolver.shared.registerAsSingleton(MaternityDataEnvironmentStore.self, MaternityDataEnvironmentStoreImpl.shared)
        Resolver.shared.registerAsSingleton(KnowledgeEnvironmentStore.self, KnowledgeEnvironmentStoreImpl.shared)

        // Reset user data
        let exp = XCTestExpectation()
        var cancelBag = CancelBag()
        userContext.resetAllUserData()
            .sink(receiveCompletion: { _ in exp.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [exp], timeout: 2.0)
    }

    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }

    func testHasHandledOnboardingFlow() throws {
        XCTAssertFalse(userContext.hasHandledOnboardingFlow)
        userContext.setHasHandledOnboardingFlow(true)
        XCTAssertTrue(userContext.hasHandledOnboardingFlow)
    }

    func testNotLoggedIn() {
        XCTAssertFalse(userContext.isAuthenticated)
        XCTAssertNil(userContext.userId)
    }

    func testLoginAndLogout() {
        XCTAssertFalse(self.userContext.hasUserRegistered) // Indicates whether at user token is stored
        performRegister()

        let exp = XCTestExpectation(description: "Failed to login and logout")
        timAuth.loginWithPassword(userId: mockUserId, password: mockPassword, storeNewRefreshToken: true)
            .sink(
                receiveCompletion: { completion in
                    switch completion {
                    case .failure:
                        XCTFail("Failed to login!")
                    case .finished:
                        self.prepareConfigurationServiceAndEnvironmentStoreData()
                        XCTAssertNotNil(MaternityDataEnvironmentStoreImpl.shared.dateOfBirthForSelectedMaternity)
                        XCTAssertFalse(KnowledgeEnvironmentStoreImpl.shared.checklists.isEmpty)

                        self.userContext.logout()
                        XCTAssertFalse(self.userContext.isAuthenticated)
                        XCTAssertTrue(self.userContext.hasUserRegistered)

                        // Test that environment stores are cleared upon logout
                        XCTAssertNil(MaternityDataEnvironmentStoreImpl.shared.dateOfBirthForSelectedMaternity)
                        XCTAssertTrue(KnowledgeEnvironmentStoreImpl.shared.checklists.isEmpty)
                    }
                    exp.fulfill()
                },
                receiveValue: { (accessToken) in
                    XCTAssertTrue(self.userContext.isAuthenticated)
            })
            .store(in: &self.store)


        wait(for: [exp], timeout: 10.0)
    }

    func testResetUserData() {
        performRegister()
        prepareConfigurationServiceAndEnvironmentStoreData()
        XCTAssertNotNil(MaternityDataEnvironmentStoreImpl.shared.dateOfBirthForSelectedMaternity)
        XCTAssertFalse(KnowledgeEnvironmentStoreImpl.shared.checklists.isEmpty)

        let exp = XCTestExpectation(description: "Failed to reset user data")
        self.userContext.resetAllUserData()
            .sink(
                receiveCompletion: { completion  in
                    switch completion {
                    case .finished:
                        XCTAssertFalse(self.userContext.isAuthenticated)
                        XCTAssertNil(self.userContext.userId)

                        // Test that environment stores are cleared upon reset
                        XCTAssertNil(MaternityDataEnvironmentStoreImpl.shared.dateOfBirthForSelectedMaternity)
                        XCTAssertTrue(KnowledgeEnvironmentStoreImpl.shared.checklists.isEmpty)
                    case .failure:
                        XCTFail()
                    }
                    exp.fulfill()
                },
                receiveValue: { }
            )
            .store(in: &self.store)

        wait(for: [exp], timeout: 10.0)
    }

    private func prepareConfigurationServiceAndEnvironmentStoreData() {
        var cancelBag = CancelBag()
        let expSetupService = XCTestExpectation()

        // Setup services for data
        configurationService.updateDateOfBirth(maternityId: "1", date: Date())
            .flatMap({ _ in
                self.configurationService.updateChecklistItem(maternityId: "1", itemId: "1-0", isChecked: true)
            })
            .sink(
                receiveCompletion: { (completion) in
                    switch completion {
                    case .finished:
                        expSetupService.fulfill()
                    case .failure(let error):
                        XCTFail(error.localizedDescription)
                    }
                }, receiveValue: { _ in })
            .store(in: &cancelBag)

        wait(for: [expSetupService], timeout: 5.0)

        // Make sure the stores has loaded the data from the services.
        let expSetupMaternityStore = XCTestExpectation()
        let expSetupKnowledgeStore = XCTestExpectation()
        MaternityDataEnvironmentStoreImpl.shared.requestUpdate()
            .sink(
                receiveCompletion: { (completion) in
                    switch completion {
                    case .finished:
                        KnowledgeEnvironmentStoreImpl.shared.fetchChecklists()
                        expSetupMaternityStore.fulfill()
                    case .failure(let error):
                        XCTFail(error.localizedDescription)
                    }
                }, receiveValue: { _ in })
            .store(in: &cancelBag)

        KnowledgeEnvironmentStoreImpl.shared.$checklists
            .sink(
                receiveCompletion: { _ in },
                receiveValue: { checklists in
                    if checklists.contains(where: { $0.completedItems == 1 }) {
                        expSetupKnowledgeStore.fulfill()
                    }
                }
            )
            .store(in: &cancelBag)

        wait(for: [expSetupMaternityStore, expSetupKnowledgeStore], timeout: 5.0)
    }

    private func performRegister() {
        let exp = XCTestExpectation(description: "Failed to reset user data")
        timAuth.performOpenIDConnectLogin(presentingViewController: UIViewController())
            .flatMap({ _ -> AnyPublisher<Void, TIMError> in
                if let refreshToken = self.timAuth.refreshToken {
                    return self.timDataStorage.storeRefreshToken(refreshToken, withNewPassword: mockPassword).mapToVoid().eraseToAnyPublisher()
                } else {
                    return Fail<Void, TIMError>(error: TIMError.storage(.encryptedStorageFailed(.unexpectedData))).eraseToAnyPublisher()
                }
            })
            .sink(
                receiveCompletion: { completion in
                    switch completion {
                    case .failure:
                        XCTFail("Failed to login!")
                    case .finished:
                        XCTAssertTrue(self.userContext.hasUserRegistered)
                        XCTAssertTrue(self.userContext.isAuthenticated)
                        XCTAssertEqual(self.userContext.userId, mockUserId)
                        exp.fulfill()
                    }
                },
                receiveValue: { accessToken in

                }
            ).store(in: &self.store)

        wait(for: [exp], timeout: 10.0)
    }
}
